package cz.upol.jj1.lab03;


import java.util.Arrays;

/**
 * Car with reasonable encapsulation of fields.
 */
class BetterCar {

    private final String plateNumber;  //nechceme dovolit zmenu RZ
    private Color color;
    private int speed;
    private String location;
    private Person[] crew;

    private final int minSpeed;
    private final int maxSpeed;


    public BetterCar(String plateNumber, String color, String location, int minSpeed, int maxSpeed) {
        this.plateNumber = plateNumber;
        this.color = new Color(color);
        this.speed = 0;
        this.location = location;
        this.crew = new Person[5];
        this.minSpeed = minSpeed;
        this.maxSpeed = maxSpeed;
    }

    /**
     * @return Car's color
     */
    public Color getColor() {
        return color;
    }

    /**
     * Sets car's color.
     * @param color
     */
    public void setColor(Color color) {
        this.color = color;
    }

    /**
     * Car's speed.
     * @return
     */
    public int getSpeed() {
        return speed;
    }

    /**
     * Sets car's speed to given value, if the value is valid.
     * @param speed
     * @return true if speed was changed, false otherwise
     */
    public boolean setSpeed(int speed) {
        if (this.minSpeed > speed || this.maxSpeed < speed) {
            return false;
        }
        this.speed = speed;
        return true;
    }

    /**
     * Car's location.
     * @return
     */
    public String getLocation() {
        return location;
    }

    /**
     * Car's plate number.
     * @return
     */
    public String getPlateNumber() {
        return plateNumber;
    }

    /**
     * A shallow copy of the car's crew.
     *
     * @return copy of the car's crew.
     */
    public Person[] getCrew() {
        return Arrays.copyOf(crew, crew.length);
    }

    /**
     * Moves car to the given location.
     * @param location
     */
    public void setLocation(String location) {
        this.location = location;
    }

    public void setLocation(String country, String city){
        this.location = country + city;
    }

    /**
     * Make a given person to disembark, if it is in the car.
     * Car is going to stop if it is moving.
     * @param person
     */
    public void disembark(Person person) {
        if (this.speed != 0) {
            this.setSpeed(0);
        }
        for (int i = 0; i < crew.length; i++) {
            if (person.equals(crew[i])) {
                crew[i] = null;
            }
        }
    }

    /**
     * Is the given person in the car?
     * @param newPerson
     * @return
     */
    public boolean isInCar(Person newPerson) {
        for (Person oldPerson : crew) {  //iterujeme pres pole (prvek po prvku)
            if (newPerson.equals(oldPerson)) {
                return true;
            }
        }
        return false;
    }


    /**
     * Lets ride faster!
     * @param speed
     * @return
     */
    public boolean speedUp(int speed) {
        int newSpeed = this.speed - speed;
        if (speed < 0 || !this.isValidSpeed(newSpeed)) {
            return false;
        }
        this.speed += speed;
        return true;
    }

    /**
     * Slow down!
     * @param speed
     * @return
     */
    public boolean slowDown(int speed) {
        int newSpeed = this.speed - speed;
        if (speed < 0 || !this.isValidSpeed(newSpeed)) {
            return false;
        }
        this.speed = newSpeed;
        return true;
    }

    /**
     * Move to the given location.
     * @param location
     */
    public void goTo(String location) {
        this.setLocation(location);
    }

    /**
     * Is the given speed valid value for the car?
     * @param value
     * @return
     */
    private boolean isValidSpeed(int value) {  //vsimnete si, ze je to private (muzu volat jen z kodu tridy)
        return value >= minSpeed && value < maxSpeed;
    }

}
